@@ -20,6 +20,7 @@ from ..base import tokenize
 from ..compatibility import unicode, apply
 from .. import array as da
 from ..async import get_sync
+from ..context import _globals
 from ..delayed import Delayed, delayed
 import dask.multiprocessing
 
@@ -189,7 +190,6 @@ def from_bcolz(x, chunksize=None, categorize=True, index=None, lock=lock,
 
     Parameters
     ----------
-
     x : bcolz.ctable
         Input data
     chunksize : int, optional
@@ -204,7 +204,6 @@ def from_bcolz(x, chunksize=None, categorize=True, index=None, lock=lock,
 
     See Also
     --------
-
     from_array: more generic function not optimized for bcolz
     """
     if lock is True:
@@ -253,7 +252,7 @@ def from_bcolz(x, chunksize=None, categorize=True, index=None, lock=lock,
         assert index in x.names
         a = da.from_array(x[index], chunks=(chunksize * len(x.names),))
         q = np.linspace(0, 100, len(x) // chunksize + 2)
-        divisions = da.percentile(a, q).compute()
+        divisions = tuple(da.percentile(a, q).compute())
         return set_partition(result, index, divisions, **kwargs)
     else:
         return result
@@ -264,7 +263,6 @@ def dataframe_from_ctable(x, slc, columns=None, categories=None, lock=lock):
 
     Parameters
     ----------
-
     x: bcolz.ctable
     slc: slice
     columns: list of column names or None
@@ -450,6 +448,12 @@ def to_hdf(df, path_or_buf, key, mode='a', append=False, complevel=0,
     if '*' in key:
         single_node = False
 
+    if 'format' in kwargs:
+        warn("argument 'format' is ignored, only 'table' is supported.")
+
+    if mode not in ('a', 'w', 'r+'):
+        raise ValueError("Mode must be one of 'a', 'w' or 'r+'")
+
     if name_function is None:
         name_function = build_name_function(df.npartitions - 1)
 
@@ -461,15 +465,24 @@ def to_hdf(df, path_or_buf, key, mode='a', append=False, complevel=0,
             warn("To preserve order between partitions name_function "
                  "must preserve the order of its input")
 
-    # handle lock default based on wether we're writing to a single entity
+    # If user did not specify scheduler and write is sequential default to the
+    # sequential scheduler. otherwise let the _get method choose the scheduler
+    if get is None and not 'get' in _globals and single_node and single_file:
+        get = get_sync
+
+    # handle lock default based on whether we're writing to a single entity
+    _actual_get = get or _globals.get('get') or df._default_get
     if lock is None:
-        if not single_node or not single_file:
+        if not single_node:
+            lock = True
+        elif not single_file and not _actual_get is dask.multiprocessing.get:
+            # if we're writing to multiple files with the multiprocessing
+            # scheduler we don't need to lock
             lock = True
         else:
             lock = False
 
     if lock is True:
-        _actual_get = get if get else dask.get
         if _actual_get == dask.multiprocessing.get:
             lock = multiprocessing.Manager().Lock()
         else:
@@ -527,12 +540,12 @@ and stopping index per file, or starting and stopping index of the global
 dataset."""
 
 def _read_single_hdf(path, key, start=0, stop=None, columns=None,
-                     chunksize=int(1e6), lock=None):
+                     chunksize=int(1e6), sorted_index=False, lock=None):
     """
     Read a single hdf file into a dask.dataframe. Used for each file in
     read_hdf.
     """
-    def get_keys_and_stops(path, key, stop):
+    def get_keys_stops_divisions(path, key, stop, sorted_index):
         """
         Get the "keys" or group identifiers which match the given key, which
         can contain wildcards. This uses the hdf file identified by the
@@ -542,18 +555,28 @@ def _read_single_hdf(path, key, start=0, stop=None, columns=None,
         with pd.HDFStore(path) as hdf:
             keys = [k for k in hdf.keys() if fnmatch(k, key)]
             stops = []
+            divisions = []
             for k in keys:
                 storer = hdf.get_storer(k)
                 if storer.format_type != 'table':
                     raise TypeError(dont_use_fixed_error_message)
                 if stop is None:
                     stops.append(storer.nrows)
+                elif stop > storer.nrows:
+                    raise ValueError("Stop keyword exceeds dataset number "
+                                     "of rows ({})".format(storer.nrows))
                 else:
                     stops.append(stop)
-        return keys, stops
+                if sorted_index:
+                    division_start = storer.read_column('index', start=0, stop=1)[0]
+                    division_end = storer.read_column('index', start=storer.nrows-1, stop=storer.nrows)[0]
+                    divisions.append([division_start, division_end])
+                else:
+                    divisions.append(None)
+        return keys, stops, divisions
 
 
-    def one_path_one_key(path, key, start, stop, columns, chunksize, lock):
+    def one_path_one_key(path, key, start, stop, columns, chunksize, division, lock):
         """
         Get the data frame corresponding to one path and one key (which should
         not contain any wildcards).
@@ -563,24 +586,35 @@ def _read_single_hdf(path, key, start=0, stop=None, columns=None,
             empty = empty[columns]
 
         token = tokenize((path, os.path.getmtime(path), key, start,
-                          stop, empty, chunksize))
+                          stop, empty, chunksize, division))
         name = 'read-hdf-' + token
 
-        dsk = dict(((name, i), (_pd_read_hdf, path, key, lock,
-                                 {'start': s,
-                                  'stop': s + chunksize,
-                                  'columns': empty.columns}))
-                    for i, s in enumerate(range(start, stop, chunksize)))
+        if start >= stop:
+            raise ValueError("Start row number ({}) is above or equal to stop "
+                             "row number ({})".format(start, stop))
+
+        if division:
+            dsk = {(name, 0): (_pd_read_hdf, path, key, lock,
+                                 {'columns': empty.columns})}
+
+            divisions = division
+        else:
+            dsk = dict(((name, i), (_pd_read_hdf, path, key, lock,
+                                     {'start': s,
+                                      'stop': s + chunksize,
+                                      'columns': empty.columns}))
+                        for i, s in enumerate(range(start, stop, chunksize)))
+
+            divisions = [None]  * (len(dsk) + 1)
 
-        divisions = [None] * (len(dsk) + 1)
         return DataFrame(dsk, name, empty, divisions)
 
-    keys, stops = get_keys_and_stops(path, key, stop)
+    keys, stops, divisions = get_keys_stops_divisions(path, key, stop, sorted_index)
     if (start != 0 or stop is not None) and len(keys) > 1:
         raise NotImplementedError(read_hdf_error_msg)
     from .multi import concat
-    return concat([one_path_one_key(path, k, start, s, columns, chunksize, lock)
-                   for k, s in zip(keys, stops)])
+    return concat([one_path_one_key(path, k, start, s, columns, chunksize, d, lock)
+                   for k, s, d in zip(keys, stops, divisions)])
 
 
 def _pd_read_hdf(path, key, lock, kwargs):
@@ -597,7 +631,7 @@ def _pd_read_hdf(path, key, lock, kwargs):
 
 @wraps(pd.read_hdf)
 def read_hdf(pattern, key, start=0, stop=None, columns=None,
-             chunksize=1000000, lock=True):
+             chunksize=1000000, sorted_index=False, lock=True):
     """
     Read hdf files into a dask dataframe. Like pandas.read_hdf, except it we
     can read multiple files, and read multiple keys from the same file by using
@@ -612,7 +646,8 @@ def read_hdf(pattern, key, start=0, stop=None, columns=None,
         stop at
     columns : optional, a list of columns that if not None, will limit the
         return columns
-    chunksize : optional, nrows to include in iteration, return an iterator
+    chunksize : optional, positive integer
+        maximal number of rows per partition
 
     Returns
     -------
@@ -639,9 +674,15 @@ def read_hdf(pattern, key, start=0, stop=None, columns=None,
     paths = sorted(glob(pattern))
     if (start != 0 or stop is not None) and len(paths) > 1:
         raise NotImplementedError(read_hdf_error_msg)
+    if chunksize <= 0:
+        raise ValueError("Chunksize must be a positive integer")
+    if (start != 0 or stop is not None) and sorted_index:
+        raise ValueError("When assuming pre-partitioned data, data must be "
+                         "read in its entirety using the same chunksizes")
     from .multi import concat
     return concat([_read_single_hdf(path, key, start=start, stop=stop,
                                     columns=columns, chunksize=chunksize,
+                                    sorted_index=sorted_index,
                                     lock=lock)
                    for path in paths])
 
@@ -735,16 +776,20 @@ def to_csv(df, filename, name_function=None, compression=None, get=None, compute
         return delayed([Delayed(key, [dsk]) for key in keys])
 
 
+def _df_to_bag(df, index=False):
+    if isinstance(df, pd.DataFrame):
+        return list(map(tuple, df.itertuples(index)))
+    elif isinstance(df, pd.Series):
+        return list(df.iteritems()) if index else list(df)
+
+
 def to_bag(df, index=False):
     from ..bag.core import Bag
-    if isinstance(df, DataFrame):
-        func = lambda df: list(df.itertuples(index))
-    elif isinstance(df, Series):
-        func = (lambda df: list(df.iteritems())) if index else list
-    else:
+    if not isinstance(df, (DataFrame, Series)):
         raise TypeError("df must be either DataFrame or Series")
     name = 'to_bag-' + tokenize(df, index)
-    dsk = dict(((name, i), (func, block)) for (i, block) in enumerate(df._keys()))
+    dsk = dict(((name, i), (_df_to_bag, block, index))
+               for (i, block) in enumerate(df._keys()))
     dsk.update(df._optimize(df.dask, df._keys()))
     return Bag(dsk, name, df.npartitions)
 
