@@ -67,7 +67,7 @@ from ..compatibility import apply
 from .core import (_Frame, Scalar, DataFrame, map_partitions,
                    Index, _maybe_from_pandas)
 from .io import from_pandas
-from .shuffle import shuffle
+from .shuffle import shuffle, rearrange_by_divisions
 
 
 def bound(seq, left, right):
@@ -242,8 +242,11 @@ def _pdmerge(left, right, how, left_on, right_on,
     return result
 
 
+shuffle_func = shuffle  # name sometimes conflicts with keyword argument
+
+
 def hash_join(lhs, left_on, rhs, right_on, how='inner',
-              npartitions=None, suffixes=('_x', '_y')):
+              npartitions=None, suffixes=('_x', '_y'), shuffle=None):
     """ Join two DataFrames on particular columns with hash join
 
     This shuffles both datasets on the joined column and then performs an
@@ -254,8 +257,8 @@ def hash_join(lhs, left_on, rhs, right_on, how='inner',
     if npartitions is None:
         npartitions = max(lhs.npartitions, rhs.npartitions)
 
-    lhs2 = shuffle(lhs, left_on, npartitions)
-    rhs2 = shuffle(rhs, right_on, npartitions)
+    lhs2 = shuffle_func(lhs, left_on, npartitions=npartitions, shuffle=shuffle)
+    rhs2 = shuffle_func(rhs, right_on, npartitions=npartitions, shuffle=shuffle)
 
     if isinstance(left_on, Index):
         left_on = None
@@ -284,8 +287,8 @@ def hash_join(lhs, left_on, rhs, right_on, how='inner',
     if isinstance(right_on, list):
         right_on = (list, tuple(right_on))
 
-    token = tokenize(lhs, left_on, rhs, right_on, left_index, right_index,
-                     how, npartitions, suffixes)
+    token = tokenize(lhs2, left_on, rhs2, right_on, left_index, right_index,
+                     how, npartitions, suffixes, shuffle)
     name = 'hash-join-' + token
 
     dsk = dict(((name, i), (merger, (lhs2._name, i), (rhs2._name, i),
@@ -379,8 +382,11 @@ def concat_indexed_dataframes(dfs, axis=0, join='outer'):
 
 def merge(left, right, how='inner', on=None, left_on=None, right_on=None,
           left_index=False, right_index=False, suffixes=('_x', '_y'),
-          npartitions=None):
-
+          npartitions=None, shuffle=None, max_branch=None):
+    for o in [on, left_on, right_on]:
+        if isinstance(o, _Frame):
+            raise NotImplementedError(
+                "Dask collections not currently allowed in merge columns")
     if not on and not left_on and not right_on and not left_index and not right_index:
         on = [c for c in left.columns if c in right.columns]
         if not on:
@@ -413,19 +419,43 @@ def merge(left, right, how='inner', on=None, left_on=None, right_on=None,
 
     # Both sides are now dd.DataFrame or dd.Series objects
 
-    if left_index and right_index:  # Do indexed join
+    # Both sides indexed
+    if (left_index and left.known_divisions and
+        right_index and right.known_divisions):  # Do indexed join
         return join_indexed_dataframes(left, right, how=how,
                                        lsuffix=suffixes[0], rsuffix=suffixes[1])
 
+    # Single partition on one side
     elif (left.npartitions == 1 and how in ('inner', 'right') or
           right.npartitions == 1 and how in ('inner', 'left')):
         return single_partition_join(left, right, how=how, right_on=right_on,
                 left_on=left_on, left_index=left_index,
                 right_index=right_index, suffixes=suffixes)
+
+    # One side is indexed, the other not
+    elif (left_index and left.known_divisions and not right_index or
+          right_index and right.known_divisions and not left_index):
+        left_empty = left._pd_nonempty
+        right_empty = right._pd_nonempty
+        dummy = pd.merge(left_empty, right_empty, how=how, on=on, left_on=left_on,
+                right_on=right_on, left_index=left_index, right_index=right_index,
+                suffixes=suffixes)
+        if left_index and left.known_divisions:
+            right = rearrange_by_divisions(right, right_on, left.divisions,
+                    max_branch, shuffle=shuffle)
+            left = left.clear_divisions()
+        elif right_index and right.known_divisions:
+            left = rearrange_by_divisions(left, left_on, right.divisions,
+                    max_branch, shuffle=shuffle)
+            right = right.clear_divisions()
+        return map_partitions(pd.merge, dummy, left, right, how=how, on=on,
+                left_on=left_on, right_on=right_on, left_index=left_index,
+                right_index=right_index, suffixes=suffixes)
+    # Catch all hash join
     else:
         return hash_join(left, left.index if left_index else left_on,
                          right, right.index if right_index else right_on,
-                         how, npartitions, suffixes)
+                         how, npartitions, suffixes, shuffle=shuffle)
 
 
 ###############################################################
