@@ -110,6 +110,9 @@ def test_attributes():
 
     df = dd.from_pandas(pd.DataFrame({'a b c': [1, 2, 3]}), npartitions=2)
     assert 'a b c' not in dir(df)
+    df = dd.from_pandas(pd.DataFrame({'a': [1, 2], 5: [1, 2]}), npartitions=2)
+    assert 'a' in dir(df)
+    assert 5 not in dir(df)
 
 
 def test_column_names():
@@ -523,36 +526,36 @@ def test_set_partition_compute():
     assert len(d4.dask) > len(d5.dask)
 
 
-def test_get_division():
+def test_get_partition():
     pdf = pd.DataFrame(np.random.randn(10, 5), columns=list('abcde'))
     ddf = dd.from_pandas(pdf, 3)
     assert ddf.divisions == (0, 4, 8, 9)
 
     # DataFrame
-    div1 = ddf.get_division(0)
+    div1 = ddf.get_partition(0)
     assert isinstance(div1, dd.DataFrame)
     assert eq(div1, pdf.loc[0:3])
-    div2 = ddf.get_division(1)
+    div2 = ddf.get_partition(1)
     assert eq(div2, pdf.loc[4:7])
-    div3 = ddf.get_division(2)
+    div3 = ddf.get_partition(2)
     assert eq(div3, pdf.loc[8:9])
     assert len(div1) + len(div2) + len(div3) == len(pdf)
 
     # Series
-    div1 = ddf.a.get_division(0)
+    div1 = ddf.a.get_partition(0)
     assert isinstance(div1, dd.Series)
     assert eq(div1, pdf.a.loc[0:3])
-    div2 = ddf.a.get_division(1)
+    div2 = ddf.a.get_partition(1)
     assert eq(div2, pdf.a.loc[4:7])
-    div3 = ddf.a.get_division(2)
+    div3 = ddf.a.get_partition(2)
     assert eq(div3, pdf.a.loc[8:9])
     assert len(div1) + len(div2) + len(div3) == len(pdf.a)
 
     with tm.assertRaises(ValueError):
-        ddf.get_division(-1)
+        ddf.get_partition(-1)
 
     with tm.assertRaises(ValueError):
-        ddf.get_division(3)
+        ddf.get_partition(3)
 
 
 def test_ndim():
@@ -1155,22 +1158,23 @@ def test_repartition_on_pandas_dataframe():
 
 
 def test_repartition_npartitions():
-    df = pd.DataFrame({'x': [1, 2, 3, 4, 5, 6], 'y': list('abdabd')},
-                      index=[10, 20, 30, 40, 50, 60])
-    for n in [1, 2, 4, 5]:
-        for k in [1, 2, 4, 5]:
-            if k > n:
-                continue
-            a = dd.from_pandas(df, npartitions=n)
-            k = min(a.npartitions, k)
+    for use_index in (True, False):
+        df = pd.DataFrame({'x': [1, 2, 3, 4, 5, 6], 'y': list('abdabd')},
+                          index=[10, 20, 30, 40, 50, 60])
+        for n in [1, 2, 4, 5]:
+            for k in [1, 2, 4, 5]:
+                if k > n:
+                    continue
+                a = dd.from_pandas(df, npartitions=n, sort=use_index)
+                k = min(a.npartitions, k)
 
-            b = a.repartition(npartitions=k)
-            eq(a, b)
-            assert b.npartitions == k
+                b = a.repartition(npartitions=k)
+                eq(a, b)
+                assert b.npartitions == k
 
-    a = dd.from_pandas(df, npartitions=1)
-    with pytest.raises(ValueError):
-        a.repartition(npartitions=5)
+        a = dd.from_pandas(df, npartitions=1)
+        with pytest.raises(ValueError):
+            a.repartition(npartitions=5)
 
 
 def test_embarrassingly_parallel_operations():
