@@ -2,9 +2,11 @@ import gzip
 import pandas as pd
 import numpy as np
 import pandas.util.testing as tm
+import sys
 import os
 import dask
 import pytest
+from distutils.version import LooseVersion
 from threading import Lock
 import shutil
 from time import sleep
@@ -13,8 +15,9 @@ import threading
 import dask.array as da
 import dask.dataframe as dd
 from dask.dataframe.io import (from_array, from_bcolz, from_dask_array)
+from dask.delayed import Delayed
 
-from dask.utils import filetext, filetexts, tmpfile, tmpdir
+from dask.utils import filetext, filetexts, tmpfile, tmpdir, dependency_depth
 from dask.async import get_sync
 
 from dask.dataframe.utils import eq
@@ -601,9 +604,11 @@ def test_from_dask_array_struct_dtype():
               pd.DataFrame(x, columns=['b', 'a']))
 
 
-@pytest.mark.xfail(reason="bloscpack BLOSC_MAX_BUFFERSIZE")
 def test_to_castra():
     pytest.importorskip('castra')
+    blosc = pytest.importorskip('blosc')
+    if LooseVersion(blosc.__version__) == '1.3.0':
+        pytest.skip()
     df = pd.DataFrame({'x': ['a', 'b', 'c', 'd'],
                        'y': [2, 3, 4, 5]},
                       index=pd.Index([1., 2., 3., 4.], name='ind'))
@@ -629,16 +634,28 @@ def test_to_castra():
     finally:
         c.drop()
 
-    dsk, keys = a.to_castra(compute=False)
-    assert isinstance(dsk, dict)
-    assert isinstance(keys, list)
-    c, last = keys
-    assert last[1] == a.npartitions - 1
+    delayed = a.to_castra(compute=False)
+    assert isinstance(delayed, Delayed)
+    c = delayed.compute()
+    try:
+        tm.assert_frame_equal(c[:], df)
+    finally:
+        c.drop()
 
+    # make sure compute=False preserves the same interface
+    c1 = a.to_castra(compute=True)
+    c2 = a.to_castra(compute=False).compute()
+    try:
+        tm.assert_frame_equal(c1[:], c2[:])
+    finally:
+        c1.drop()
+        c2.drop()
 
-@pytest.mark.xfail(reason="bloscpack BLOSC_MAX_BUFFERSIZE")
 def test_from_castra():
     pytest.importorskip('castra')
+    blosc = pytest.importorskip('blosc')
+    if LooseVersion(blosc.__version__) == '1.3.0':
+        pytest.skip()
     df = pd.DataFrame({'x': ['a', 'b', 'c', 'd'],
                        'y': [2, 3, 4, 5]},
                       index=pd.Index([1., 2., 3., 4.], name='ind'))
@@ -658,13 +675,15 @@ def test_from_castra():
         del with_fn, c
 
 
-@pytest.mark.xfail(reason="bloscpack BLOSC_MAX_BUFFERSIZE")
 def test_from_castra_with_selection():
     """ Optimizations fuse getitems with load_partitions
 
     We used to use getitem for both column access and selections
     """
     pytest.importorskip('castra')
+    blosc = pytest.importorskip('blosc')
+    if LooseVersion(blosc.__version__) == '1.3.0':
+        pytest.skip()
     df = pd.DataFrame({'x': ['a', 'b', 'c', 'd'],
                        'y': [2, 3, 4, 5]},
                       index=pd.Index([1., 2., 3., 4.], name='ind'))
@@ -697,52 +716,368 @@ def test_to_hdf():
         out = pd.read_hdf(fn, '/data')
         tm.assert_frame_equal(df, out[:])
 
-    # saving to multiple datasets
+    # test compute = False
+    with tmpfile('h5') as fn:
+        r = a.to_hdf(fn, '/data', compute=False)
+        r.compute()
+        out = pd.read_hdf(fn, '/data')
+        tm.assert_frame_equal(df, out[:])
+
+
+def test_to_hdf_multiple_nodes():
+    pytest.importorskip('tables')
+    df = pd.DataFrame({'x': ['a', 'b', 'c', 'd'],
+                       'y': [1, 2, 3, 4]}, index=[1., 2., 3., 4.])
     a = dd.from_pandas(df, 2)
+    df16 = pd.DataFrame({'x': ['a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j', 'k', 'l', 'm', 'n', 'o', 'p'],
+                       'y': [1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16]},
+                            index=[1., 2., 3., 4., 5., 6., 7., 8., 9., 10., 11., 12., 13., 14., 15., 16.])
+    b = dd.from_pandas(df16, 16)
+
+    # saving to multiple nodes
     with tmpfile('h5') as fn:
         a.to_hdf(fn, '/data*')
         out = dd.read_hdf(fn, '/data*')
-        tm.assert_frame_equal(df, out.compute())
+        eq(df, out)
 
-    # saving to multiple files
-    a = dd.from_pandas(df, 2)
-    with tmpdir() as dn:
-        fn = os.path.join(dn, 'data_*.h5')
-        a.to_hdf(fn, '/data')
-        out = dd.read_hdf(fn, '/data')
-        tm.assert_frame_equal(df, out.compute())
+    # saving to multiple nodes making sure order is kept
+    with tmpfile('h5') as fn:
+        b.to_hdf(fn, '/data*')
+        out = dd.read_hdf(fn, '/data*')
+        eq(df16, out)
 
     # saving to multiple datasets with custom name_function
-    a = dd.from_pandas(df, 2)
     with tmpfile('h5') as fn:
-        a.to_hdf(fn, '/data_*', name_function=lambda i: 'a' * (i +  1))
+        a.to_hdf(fn, '/data_*', name_function=lambda i: 'a' * (i + 1))
         out = dd.read_hdf(fn, '/data_*')
-        tm.assert_frame_equal(df, out.compute())
+        eq(df, out)
 
         out = pd.read_hdf(fn, '/data_a')
         tm.assert_frame_equal(out, df.iloc[:2])
         out = pd.read_hdf(fn, '/data_aa')
         tm.assert_frame_equal(out, df.iloc[2:])
 
-    # saving to multiple files with custom name_function
+    # test multiple nodes with hdf object
+    with tmpfile('h5') as fn:
+        with pd.HDFStore(fn) as hdf:
+            b.to_hdf(hdf, '/data*')
+            out = dd.read_hdf(fn, '/data*')
+            eq(df16, out)
+
+
+def test_to_hdf_multiple_files():
+    pytest.importorskip('tables')
+    df = pd.DataFrame({'x': ['a', 'b', 'c', 'd'],
+                       'y': [1, 2, 3, 4]}, index=[1., 2., 3., 4.])
     a = dd.from_pandas(df, 2)
+    df16 = pd.DataFrame({'x': ['a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j', 'k', 'l', 'm', 'n', 'o', 'p'],
+                       'y': [1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16]},
+                            index=[1., 2., 3., 4., 5., 6., 7., 8., 9., 10., 11., 12., 13., 14., 15., 16.])
+    b = dd.from_pandas(df16, 16)
+
+    # saving to multiple files
     with tmpdir() as dn:
         fn = os.path.join(dn, 'data_*.h5')
-        a.to_hdf(fn, '/data', name_function=lambda i: 'a' * (i +  1))
+        a.to_hdf(fn, '/data')
         out = dd.read_hdf(fn, '/data')
-        tm.assert_frame_equal(df, out.compute())
+        eq(df, out)
+
+    # saving to multiple files making sure order is kept
+    with tmpdir() as dn:
+        fn = os.path.join(dn, 'data_*.h5')
+        b.to_hdf(fn, '/data')
+        out = dd.read_hdf(fn, '/data')
+        eq(df16, out)
+
+    # saving to multiple files with custom name_function
+    with tmpdir() as dn:
+        fn = os.path.join(dn, 'data_*.h5')
+        a.to_hdf(fn, '/data', name_function=lambda i: 'a' * (i + 1))
+        out = dd.read_hdf(fn, '/data')
+        eq(df, out)
 
         out = pd.read_hdf(os.path.join(dn, 'data_a.h5'), '/data')
         tm.assert_frame_equal(out, df.iloc[:2])
         out = pd.read_hdf(os.path.join(dn, 'data_aa.h5'), '/data')
         tm.assert_frame_equal(out, df.iloc[2:])
 
-    # saving to different datasets in multiple files with custom name_function
+    # test hdf object
+    with tmpfile('h5') as fn:
+        with pd.HDFStore(fn) as hdf:
+            a.to_hdf(hdf, '/data*')
+            out = dd.read_hdf(fn, '/data*')
+            eq(df, out)
+
+
+def test_to_hdf_modes_multiple_nodes():
+    pytest.importorskip('tables')
+    df = pd.DataFrame({'x': ['a', 'b', 'c', 'd'],
+                       'y': [1, 2, 3, 4]}, index=[1., 2., 3., 4.])
+
+    # appending a single partition to existing data
+    a = dd.from_pandas(df, 1)
+    with tmpfile('h5') as fn:
+        a.to_hdf(fn, '/data2')
+        a.to_hdf(fn, '/data*', mode='a')
+        out = dd.read_hdf(fn, '/data*')
+        eq(df.append(df), out)
+
+    # overwriting a file with a single partition
+    a = dd.from_pandas(df, 1)
+    with tmpfile('h5') as fn:
+        a.to_hdf(fn, '/data2')
+        a.to_hdf(fn, '/data*', mode='w')
+        out = dd.read_hdf(fn, '/data*')
+        eq(df, out)
+
+    # appending two partitions to existing data
     a = dd.from_pandas(df, 2)
+    with tmpfile('h5') as fn:
+        a.to_hdf(fn, '/data2')
+        a.to_hdf(fn, '/data*', mode='a')
+        out = dd.read_hdf(fn, '/data*')
+        eq(df.append(df), out)
+
+    # overwriting a file with two partitions
+    a = dd.from_pandas(df, 2)
+    with tmpfile('h5') as fn:
+        a.to_hdf(fn, '/data2')
+        a.to_hdf(fn, '/data*', mode='w')
+        out = dd.read_hdf(fn, '/data*')
+        eq(df, out)
+
+    # overwriting a single partition, keeping other partitions
+    a = dd.from_pandas(df, 2)
+    with tmpfile('h5') as fn:
+        a.to_hdf(fn, '/data1')
+        a.to_hdf(fn, '/data2')
+        a.to_hdf(fn, '/data*', mode='a', append=False)
+        out = dd.read_hdf(fn, '/data*')
+        eq(df.append(df), out)
+
+
+def test_to_hdf_modes_multiple_files():
+    pytest.importorskip('tables')
+    df = pd.DataFrame({'x': ['a', 'b', 'c', 'd'],
+                       'y': [1, 2, 3, 4]}, index=[1., 2., 3., 4.])
+
+    # appending a single partition to existing data
+    a = dd.from_pandas(df, 1)
+    with tmpdir() as dn:
+        fn = os.path.join(dn, 'data*')
+        a.to_hdf(os.path.join(dn, 'data2'), '/data')
+        a.to_hdf(fn, '/data', mode='a')
+        out = dd.read_hdf(fn, '/data*')
+        eq(df.append(df), out)
+
+    # appending two partitions to existing data
+    a = dd.from_pandas(df, 2)
+    with tmpdir() as dn:
+        fn = os.path.join(dn, 'data*')
+        a.to_hdf(os.path.join(dn, 'data2'), '/data')
+        a.to_hdf(fn, '/data', mode='a')
+        out = dd.read_hdf(fn, '/data')
+        eq(df.append(df), out)
+
+    # overwriting a file with two partitions
+    a = dd.from_pandas(df, 2)
+    with tmpdir() as dn:
+        fn = os.path.join(dn, 'data*')
+        a.to_hdf(os.path.join(dn, 'data1'), '/data')
+        a.to_hdf(fn, '/data', mode='w')
+        out = dd.read_hdf(fn, '/data')
+        eq(df, out)
+
+    # overwriting a single partition, keeping other partitions
+    a = dd.from_pandas(df, 2)
+    with tmpdir() as dn:
+        fn = os.path.join(dn, 'data*')
+        a.to_hdf(os.path.join(dn, 'data1'), '/data')
+        a.to_hdf(fn, '/data', mode='a', append=False)
+        out = dd.read_hdf(fn, '/data')
+        eq(df.append(df), out)
+
+
+def test_to_hdf_link_optimizations():
+    """testing dask link levels is correct by calculating the depth of the dask graph"""
+    pytest.importorskip('tables')
+    df16 = pd.DataFrame({'x': ['a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j', 'k', 'l', 'm', 'n', 'o', 'p'],
+                       'y': [1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16]},
+                            index=[1., 2., 3., 4., 5., 6., 7., 8., 9., 10., 11., 12., 13., 14., 15., 16.])
+    a = dd.from_pandas(df16, 16)
+
+    # saving to multiple hdf files, no links are needed
+    # expected layers: from_pandas, to_hdf, list = depth of 3
+    with tmpdir() as dn:
+        fn = os.path.join(dn, 'data*')
+        d = a.to_hdf(fn, '/data', compute=False)
+        assert dependency_depth(d.dask) == 3
+
+    # saving to a single hdf file with multiple nodes
+    # all subsequent nodes depend on the first
+    # expected layers: from_pandas, first to_hdf(creates file+node), subsequent to_hdfs, list = 4
+    with tmpfile() as fn:
+        d = a.to_hdf(fn, '/data*', compute=False)
+        assert dependency_depth(d.dask) == 4
+
+    # saving to a single hdf file with a single node
+    # every node depends on the previous node
+    # expected layers: from_pandas, to_hdf times npartitions(15), list = 2 + npartitions = 17
+    with tmpfile() as fn:
+        d = a.to_hdf(fn, '/data', compute=False)
+        assert dependency_depth(d.dask) == 2 + a.npartitions
+
+
+@pytest.mark.slow
+def test_to_hdf_lock_delays():
+    pytest.importorskip('tables')
+    df16 = pd.DataFrame({'x': ['a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j', 'k', 'l', 'm', 'n', 'o', 'p'],
+                       'y': [1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16]},
+                            index=[1., 2., 3., 4., 5., 6., 7., 8., 9., 10., 11., 12., 13., 14., 15., 16.])
+    a = dd.from_pandas(df16, 16)
+
+    # adding artifichial delays to make sure last tasks finish first
+    # that's a way to simulate last tasks finishing last
+    def delayed_nop(i):
+        if i[1] < 10:
+            sleep(0.1*(10-i[1]))
+        return i
+
+    # saving to multiple hdf nodes
+    with tmpfile() as fn:
+        a = a.apply(delayed_nop, axis=1, columns=a.columns)
+        a.to_hdf(fn, '/data*')
+        out = dd.read_hdf(fn, '/data*')
+        eq(df16, out)
+
+    # saving to multiple hdf files
+    # adding artifichial delays to make sure last tasks finish first
+    with tmpdir() as dn:
+        fn = os.path.join(dn, 'data*')
+        a = a.apply(delayed_nop, axis=1, columns=a.columns)
+        a.to_hdf(fn, '/data')
+        out = dd.read_hdf(fn, '/data')
+        eq(df16, out)
+
+
+def test_to_hdf_exceptions():
+    pytest.importorskip('tables')
+    df = pd.DataFrame({'x': ['a', 'b', 'c', 'd'],
+                       'y': [1, 2, 3, 4]}, index=[1., 2., 3., 4.])
+    a = dd.from_pandas(df, 1)
+
+    # triggering too many asterisks error
     with tmpdir() as dn:
         with pytest.raises(ValueError):
             fn = os.path.join(dn, 'data_*.h5')
-            a.to_hdf(fn, '/data_*', name_function=lambda i: 'a' * (i +  1))
+            a.to_hdf(fn, '/data_*')
+
+    # triggering too many asterisks error
+    with tmpfile() as fn:
+        with pd.HDFStore(fn) as hdf:
+            with pytest.raises(ValueError):
+                a.to_hdf(hdf, '/data_*_*')
+
+
+def test_to_hdf_sync():
+    pytest.importorskip('tables')
+    df = pd.DataFrame({'x': ['a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j', 'k', 'l', 'm', 'n', 'o', 'p'],
+                       'y': [1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16]},
+                            index=[1., 2., 3., 4., 5., 6., 7., 8., 9., 10., 11., 12., 13., 14., 15., 16.])
+    a = dd.from_pandas(df, 16)
+
+    # test single file single node
+    with tmpfile('h5') as fn:
+        a.to_hdf(fn, '/data', get=get_sync)
+        out = pd.read_hdf(fn, '/data')
+        eq(df, out)
+
+    # test multiple files single node
+    with tmpdir() as dn:
+        fn = os.path.join(dn, 'data_*.h5')
+        a.to_hdf(fn, '/data', get=get_sync)
+        out = dd.read_hdf(fn, '/data')
+        eq(df, out)
+
+    # test single file multiple nodes
+    with tmpfile('h5') as fn:
+        a.to_hdf(fn, '/data*', get=get_sync)
+        out = dd.read_hdf(fn, '/data*')
+        eq(df, out)
+
+
+def test_to_hdf_thread():
+    pytest.importorskip('tables')
+    df = pd.DataFrame({'x': ['a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j', 'k', 'l', 'm', 'n', 'o', 'p'],
+                       'y': [1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16]},
+                            index=[1., 2., 3., 4., 5., 6., 7., 8., 9., 10., 11., 12., 13., 14., 15., 16.])
+    a = dd.from_pandas(df, 16)
+
+    # test single file single node
+    with tmpfile('h5') as fn:
+        a.to_hdf(fn, '/data', get=dask.threaded.get)
+        out = pd.read_hdf(fn, '/data')
+        eq(df, out)
+
+    # test multiple files single node
+    with tmpdir() as dn:
+        fn = os.path.join(dn, 'data_*.h5')
+        a.to_hdf(fn, '/data', get=dask.threaded.get)
+        out = dd.read_hdf(fn, '/data')
+        eq(df, out)
+
+    # test single file multiple nodes
+    with tmpfile('h5') as fn:
+        a.to_hdf(fn, '/data*', get=dask.threaded.get)
+        out = dd.read_hdf(fn, '/data*')
+        eq(df, out)
+
+
+def test_to_hdf_process():
+    pytest.importorskip('tables')
+    df = pd.DataFrame({'x': ['a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j', 'k', 'l', 'm', 'n', 'o', 'p'],
+                       'y': [1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16]},
+                            index=[1., 2., 3., 4., 5., 6., 7., 8., 9., 10., 11., 12., 13., 14., 15., 16.])
+    a = dd.from_pandas(df, 16)
+
+    # test single file single node
+    with tmpfile('h5') as fn:
+        a.to_hdf(fn, '/data', get=dask.multiprocessing.get)
+        out = pd.read_hdf(fn, '/data')
+        eq(df, out)
+
+    # test multiple files single node
+    with tmpdir() as dn:
+        fn = os.path.join(dn, 'data_*.h5')
+        a.to_hdf(fn, '/data', get=dask.multiprocessing.get)
+        out = dd.read_hdf(fn, '/data')
+        eq(df, out)
+
+    # test single file multiple nodes
+    with tmpfile('h5') as fn:
+        a.to_hdf(fn, '/data*', get=dask.multiprocessing.get)
+        out = dd.read_hdf(fn, '/data*')
+        eq(df, out)
+
+
+@pytest.mark.skipif(sys.version_info[:2] == (3,3), reason="Python3.3 uses pytest2.7.2, w/o warns method")
+def test_to_fmt_warns():
+    pytest.importorskip('tables')
+    df16 = pd.DataFrame({'x': ['a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j', 'k', 'l', 'm', 'n', 'o', 'p'],
+                       'y': [1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16]},
+                            index=[1., 2., 3., 4., 5., 6., 7., 8., 9., 10., 11., 12., 13., 14., 15., 16.])
+    a = dd.from_pandas(df16, 16)
+
+    # testing warning when breaking order
+    with tmpfile('h5') as fn:
+        with pytest.warns(None):
+            a.to_hdf(fn, '/data*', name_function=str)
+
+    # testing warning when breaking order
+    with tmpdir() as dn:
+        with pytest.warns(None):
+            fn = os.path.join(dn, "data_*.csv")
+            a.to_csv(fn, name_function=str)
 
 
 def test_read_hdf():
@@ -772,20 +1107,112 @@ def test_read_hdf():
         assert (sorted(dd.read_hdf(fn, '/data').dask) ==
                 sorted(dd.read_hdf(fn, '/data').dask))
 
+def test_read_hdf_multiple():
+    pytest.importorskip('tables')
+    df = pd.DataFrame({'x': ['a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j', 'k', 'l', 'm', 'n', 'o', 'p'],
+                       'y': [1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16]},
+                            index=[1., 2., 3., 4., 5., 6., 7., 8., 9., 10., 11., 12., 13., 14., 15., 16.])
+    a = dd.from_pandas(df, 16)
+
+    with tmpfile('h5') as fn:
+        a.to_hdf(fn, '/data*')
+        r = dd.read_hdf(fn, '/data*', sorted_index=True)
+        assert a.npartitions == r.npartitions
+        assert a.divisions == r.divisions
+        eq(a, r)
+
+
+def test_read_hdf_start_stop_values():
+    pytest.importorskip('tables')
+    df = pd.DataFrame({'x': ['a', 'b', 'c', 'd'],
+                       'y': [1, 2, 3, 4]}, index=[1., 2., 3., 4.])
+    with tmpfile('h5') as fn:
+        df.to_hdf(fn, '/data', format='table')
+
+        with pytest.raises(ValueError) as e:
+            a = dd.read_hdf(fn, '/data', stop=10)
+        assert 'number of rows' in str(e)
+
+        with pytest.raises(ValueError) as e:
+            a = dd.read_hdf(fn, '/data', start=10)
+        assert 'is above or equal to' in str(e)
+
+        with pytest.raises(ValueError) as e:
+            a = dd.read_hdf(fn, '/data', chunksize=-1)
+        assert 'positive integer' in str(e)
+
 
 def test_to_csv():
     df = pd.DataFrame({'x': ['a', 'b', 'c', 'd'],
-                       'y': [1, 2, 3, 4]}, index=[1., 2., 3., 4.])
+                       'y': [1, 2, 3, 4]})
 
     for npartitions in [1, 2]:
         a = dd.from_pandas(df, npartitions)
         with tmpfile('csv') as fn:
+            a.to_csv(fn, index=False)
+            result = dd.read_csv(fn).compute().reset_index(drop=True)
+            eq(result, df)
+
+        with tmpfile('csv') as fn:
+            r = a.to_csv(fn, index=False, compute=False)
+            r.compute()
+            result = dd.read_csv(fn).compute().reset_index(drop=True)
+            eq(result, df)
+
+        with tmpdir() as dn:
+            fn = os.path.join(dn, 'data_*.csv')
+            a.to_csv(fn, index=False)
+            result = dd.read_csv(fn).compute().reset_index(drop=True)
+            eq(result, df)
+
+
+def test_to_csv_multiple_files_cornercases():
+    df = pd.DataFrame({'x': ['a', 'b', 'c', 'd'],
+                       'y': [1, 2, 3, 4]})
+    a = dd.from_pandas(df, 2)
+    with tmpdir() as dn:
+        with pytest.raises(ValueError):
+            fn = os.path.join(dn, "data_*_*.csv")
             a.to_csv(fn)
-            result = pd.read_csv(fn, index_col=0)
-            tm.assert_frame_equal(result, df)
+
+    df16 = pd.DataFrame({'x': ['a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j', 'k', 'l', 'm', 'n', 'o', 'p'],
+                       'y': [1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16]})
+    a = dd.from_pandas(df16, 16)
+    with tmpdir() as dn:
+        fn = os.path.join(dn, 'data_*.csv')
+        a.to_csv(fn, index=False)
+        result = dd.read_csv(fn).compute().reset_index(drop=True)
+        eq(result, df16)
+
+    # test handling existing files when links are optimized out
+    a = dd.from_pandas(df, 2)
+    with tmpdir() as dn:
+        fn = os.path.join(dn, 'data_1.csv')
+        a.to_csv(fn, index=False)
+        fn = os.path.join(dn, 'data_*.csv')
+        a.to_csv(fn, mode='w', index=False)
+        result = dd.read_csv(fn).compute().reset_index(drop=True)
+        eq(result, df)
+
+    # test handling existing files when links are optimized out
+    a = dd.from_pandas(df16, 16)
+    with tmpdir() as dn:
+        fn = os.path.join(dn, 'data_01.csv')
+        a.to_csv(fn, index=False)
+        fn = os.path.join(dn, 'data_*.csv')
+        a.to_csv(fn, mode='w', index=False)
+        result = dd.read_csv(fn).compute().reset_index(drop=True)
+        eq(result, df16)
+
+    # test handling existing files when mode isn't 'w'
+    a = dd.from_pandas(df, 2)
+    with tmpdir() as dn:
+        fn = os.path.join(dn, 'data_*.csv')
+        with pytest.raises(ValueError):
+            a.to_csv(fn, mode='a')
 
 
-@pytest.mark.xfail(reason="bloscpack BLOSC_MAX_BUFFERSIZE")
+@pytest.mark.xfail(reason="to_csv does not support compression")
 def test_to_csv_gzip():
     df = pd.DataFrame({'x': ['a', 'b', 'c', 'd'],
                        'y': [1, 2, 3, 4]}, index=[1., 2., 3., 4.])
@@ -877,10 +1304,10 @@ def test_to_bag():
                      index=pd.Index([1., 2., 3., 4.], name='ind'))
     ddf = dd.from_pandas(a, 2)
 
-    assert ddf.to_bag().compute(get=get_sync) == list(a.itertuples(False))
-    assert ddf.to_bag(True).compute(get=get_sync) == list(a.itertuples(True))
-    assert ddf.x.to_bag(True).compute(get=get_sync) == list(a.x.iteritems())
-    assert ddf.x.to_bag().compute(get=get_sync) == list(a.x)
+    assert ddf.to_bag().compute() == list(a.itertuples(False))
+    assert ddf.to_bag(True).compute() == list(a.itertuples(True))
+    assert ddf.x.to_bag(True).compute() == list(a.x.iteritems())
+    assert ddf.x.to_bag().compute() == list(a.x)
 
 
 @pytest.mark.xfail(reason='we might want permissive behavior here')
@@ -1080,13 +1507,13 @@ def test_read_csv_sep():
 
 
 def test_to_hdf_kwargs():
+    pytest.importorskip('tables')
     df = pd.DataFrame({'A': ['a', 'aaaa']})
     ddf = dd.from_pandas(df, npartitions=2)
-    ddf.to_hdf('tst.h5', 'foo4', format='table', min_itemsize=4)
-
-    df2 = pd.read_hdf('tst.h5', 'foo4')
-
-    tm.assert_frame_equal(df, df2)
+    with tmpfile('h5') as fn:
+        ddf.to_hdf(fn, 'foo4', format='table', min_itemsize=4)
+        df2 = pd.read_hdf(fn, 'foo4')
+        tm.assert_frame_equal(df, df2)
 
 
 def test_read_csv_slash_r():
@@ -1101,3 +1528,15 @@ def test_read_csv_singleton_dtype():
     with filetext(data, mode='wb') as fn:
         eq(pd.read_csv(fn, dtype=float),
            dd.read_csv(fn, dtype=float))
+
+
+def test_from_pandas_small():
+    for sort in [True, False]:
+        for i in [0, 2]:
+            df = pd.DataFrame({'x': [0] * i})
+            ddf = dd.from_pandas(df, npartitions=5, sort=sort)
+            eq(df, ddf)
+
+            s = pd.Series([0] * i, name='x')
+            ds = dd.from_pandas(s, npartitions=5, sort=sort)
+            eq(s, ds)
