@@ -1,13 +1,27 @@
 from __future__ import absolute_import, division, print_function
 
+from distutils.version import LooseVersion
+
 from collections import Iterator
+import sys
 
 import numpy as np
 import pandas as pd
+import pandas.util.testing as tm
+from pandas.core.common import is_datetime64tz_dtype
 import toolz
 
 from dask.async import get_sync
 
+PANDAS_VERSION = LooseVersion(pd.__version__)
+
+# Pandas <0.18.1 doesn't have the `is_extension_type` function
+if PANDAS_VERSION > '0.18.0':
+    is_extension_type = pd.core.common.is_extension_type
+else:
+    def is_extension_type(x):
+        return False
+
 
 def shard_df_on_index(df, divisions):
     """ Shard a DataFrame by ranges on its index
@@ -83,25 +97,63 @@ def unique(divisions):
         return np.unique(divisions)
     if isinstance(divisions, pd.Categorical):
         return pd.Categorical.from_codes(np.unique(divisions.codes),
-                divisions.categories, divisions.ordered)
+            divisions.categories, divisions.ordered)
     if isinstance(divisions, (tuple, list, Iterator)):
         return tuple(toolz.unique(divisions))
     raise NotImplementedError()
 
 
+_simple_fake_mapping = {
+    'b': True,   # boolean
+    'i': -1,   # signed int
+    'u': 1,   # unsigned int
+    'f': 1.0,   # float
+    'c': complex(1, 0),   # complex
+    'S': b'foo',   # bytestring
+    'U': u'foo',   # unicode string
+    'V': np.array([b' '], dtype=np.void)[0],  # void
+    'M': np.datetime64('1970-01-01'),
+    'm': np.timedelta64(1, 'D'),
+}
+
+
+def nonempty_sample_df(empty):
+    """ Create a dataframe from the given empty dataframe that contains one
+    row of fake data (generated from the empty dataframe's dtypes).
+    """
+    nonempty = {}
+    idx = pd.RangeIndex(start=0, stop=1, step=1)
+    for key, dtype in empty.dtypes.iteritems():
+        if is_datetime64tz_dtype(dtype):
+            entry = pd.Timestamp('1970-01-01', tz='America/New_York')
+        elif pd.core.common.is_categorical_dtype(dtype):
+            accessor = empty[key].cat
+            example = accessor.categories[0]
+            cat = pd.Categorical([example], categories=accessor.categories,
+                                 ordered=accessor.ordered)
+            entry = pd.Series(cat, name=key)
+        elif dtype.kind in _simple_fake_mapping:
+            entry = _simple_fake_mapping[dtype.kind]
+        elif is_extension_type(dtype):
+            raise TypeError("Can't handle extension dtype: {}".format(dtype))
+        elif dtype.name == 'object':
+            entry = 'foo'
+        else:
+            raise TypeError("Can't handle dtype: {}".format(dtype))
+
+        if not isinstance(entry, pd.Series):
+            entry = pd.Series([entry], name=key, index=idx)
+
+        nonempty[key] = entry
+
+    df = pd.DataFrame(nonempty)
+    return df
+
+
 ###############################################################
 # Testing
 ###############################################################
 
-import pandas.util.testing as tm
-from distutils.version import LooseVersion
-PANDAS_VERSION = LooseVersion(pd.__version__)
-
-if PANDAS_VERSION >= LooseVersion('0.17.0'):
-    PANDAS_0170 = True
-else:
-    PANDAS_0170 = False
-
 
 def _check_dask(dsk, check_names=True):
     import dask.dataframe as dd
@@ -145,22 +197,20 @@ def _check_dask(dsk, check_names=True):
 def _maybe_sort(a):
     # sort by value, then index
     try:
-        if PANDAS_0170:
-            if isinstance(a, pd.DataFrame):
-                a = a.sort_values(by=a.columns.tolist())
-            else:
-                a = a.sort_values()
+        if isinstance(a, pd.DataFrame):
+            a = a.sort_values(by=a.columns.tolist())
         else:
-            if isinstance(a, pd.DataFrame):
-                a = a.sort(columns=a.columns.tolist())
-            else:
-                a = a.order()
+            a = a.sort_values()
     except (TypeError, IndexError, ValueError):
         pass
     return a.sort_index()
 
 
 def eq(a, b, check_names=True, **kwargs):
+    assert_divisions(a)
+    assert_divisions(b)
+    assert_sane_keynames(a)
+    assert_sane_keynames(b)
     a = _check_dask(a, check_names=check_names)
     b = _check_dask(b, check_names=check_names)
     if isinstance(a, pd.DataFrame):
@@ -196,3 +246,35 @@ def assert_dask_graph(dask, label):
     else:
         msg = "given dask graph doesn't contan label: {0}"
         raise AssertionError(msg.format(label))
+
+
+def assert_divisions(ddf):
+    if not hasattr(ddf, 'divisions'):
+        return
+    if not hasattr(ddf, 'index'):
+        return
+    if not ddf.known_divisions:
+        return
+
+    results = get_sync(ddf.dask, ddf._keys())
+    for i, df in enumerate(results[:-1]):
+        if len(df):
+            assert df.index.min() >= ddf.divisions[i]
+            assert df.index.max() < ddf.divisions[i + 1]
+
+    if len(results[-1]):
+        assert results[-1].index.min() >= ddf.divisions[-2]
+        assert results[-1].index.max() <= ddf.divisions[-1]
+
+
+def assert_sane_keynames(ddf):
+    if not hasattr(ddf, 'dask'):
+        return
+    for k in ddf.dask.keys():
+        while isinstance(k, tuple):
+            k = k[0]
+        assert isinstance(k, (str, bytes))
+        assert len(k) < 100
+        assert ' ' not in k
+        if sys.version_info[0] >= 3:
+            assert k.split('-')[0].isidentifier()
