@@ -31,7 +31,7 @@ def unzip(ls, nout):
     return out
 
 
-def to_task_dasks(expr):
+def to_task_dasks(expr, **kwargs):
     """Normalize a python object and extract all sub-dasks.
 
     - Replace ``Delayed`` with their keys
@@ -56,7 +56,7 @@ def to_task_dasks(expr):
     >>> b = value(2, 'b')
     >>> task, dasks = to_task_dasks([a, b, 3])
     >>> task # doctest: +SKIP
-    ['a', 'b', 3]
+    (list, ['a', 'b', 3])
     >>> dasks # doctest: +SKIP
     [{'a': 1}, {'b': 2}]
 
@@ -66,6 +66,7 @@ def to_task_dasks(expr):
     >>> dasks # doctest: +SKIP
     [{'a': 1}, {'b': 2}]
     """
+    is_top = kwargs.get('is_top', True)
     if isinstance(expr, Delayed):
         return expr.key, expr._dasks
     if isinstance(expr, base.Base):
@@ -77,16 +78,17 @@ def to_task_dasks(expr):
     if isinstance(expr, tuple) and type(expr) != tuple:
         return expr, []
     if isinstance(expr, (Iterator, list, tuple, set)):
-        args, dasks = unzip(map(to_task_dasks, expr), 2)
+        args, dasks = unzip((to_task_dasks(e, is_top=False) for e in expr), 2)
         args = list(args)
         dasks = flat_unique(dasks)
         # Ensure output type matches input type
-        if isinstance(expr, (tuple, set)):
+        if is_top or isinstance(expr, (tuple, set)):
             return (type(expr), args), dasks
         else:
             return args, dasks
     if isinstance(expr, dict):
-        args, dasks = to_task_dasks([[k, v] for k, v in expr.items()])
+        args, dasks = to_task_dasks([[k, v] for k, v in expr.items()],
+                                    is_top=False)
         return (dict, args), dasks
     return expr, []
 
@@ -104,7 +106,7 @@ def tokenize(*args, **kwargs):
         unique identifier is always used.
     """
     if kwargs.pop('pure', False):
-        return base.tokenize(*args)
+        return base.tokenize(*args, **kwargs)
     else:
         return str(uuid.uuid4())
 
@@ -121,7 +123,8 @@ def delayed(obj, name=None, pure=False):
     obj : object
         The function or object to wrap
     name : string or hashable, optional
-        The key to use in the underlying graph. Defaults to hashing content.
+        The key to use in the underlying graph for the wrapped object. Defaults
+        to hashing content.
     pure : bool, optional
         Indicates whether calling the resulting ``Delayed`` object is a pure
         operation. If True, arguments to the call are hashed to produce
@@ -174,14 +177,38 @@ def delayed(obj, name=None, pure=False):
     >>> out1.key == out2.key
     True
 
+    The key name of the result of calling a delayed object is determined by
+    hashing the arguments by default. To explicitly set the name, you can use
+    the ``dask_key_name`` keyword when calling the function:
+
+    >>> add(1, 2)    # doctest: +SKIP
+    Delayed('add-3dce7c56edd1ac2614add714086e950f')
+    >>> add(1, 2, dask_key_name='three')
+    Delayed('three')
+
+    Note that objects with the same key name are assumed to have the same
+    result. If you set the names explicitly you should make sure your key names
+    are different for different results.
+
+    >>> add(1, 2, dask_key_name='three')
+    >>> add(2, 1, dask_key_name='three')
+    >>> add(2, 2, dask_key_name='four')
+
     ``delayed`` can also be applied to objects to make operations on them lazy:
 
     >>> a = delayed([1, 2, 3])
-    >>> type(a) == Delayed
+    >>> isinstance(a, Delayed)
     True
     >>> a.compute()
     [1, 2, 3]
 
+    The key name of a delayed object is hashed by default. To explicitly set
+    the name, you can use the ``name`` keyword:
+
+    >>> a = delayed([1, 2, 3], name='mylist')
+    >>> a
+    Delayed('mylist')
+
     Delayed results act as a proxy to the underlying object. Many operators
     are supported:
 
@@ -207,6 +234,13 @@ def delayed(obj, name=None, pure=False):
 
     >>> a.count(2, pure=True).key == a.count(2, pure=True).key
     True
+
+    As with function calls, method calls also support the ``dask_key_name``
+    keyword:
+
+    >>> a.count(2, dask_key_name="count_2")
+    Delayed("count_2")
+
     """
     if isinstance(obj, Delayed):
         return obj
@@ -216,7 +250,8 @@ def delayed(obj, name=None, pure=False):
     if not dasks:
         return DelayedLeaf(obj, pure=pure, name=name)
     else:
-        name = name or '%s-%s' % (type(obj).__name__, tokenize(task))
+        if not name:
+            name = '%s-%s' % (type(obj).__name__, tokenize(task, pure=pure))
         dasks.append({name: task})
         return Delayed(name, dasks)
 
@@ -313,7 +348,11 @@ class Delayed(base.Base):
 
     def __call__(self, *args, **kwargs):
         pure = kwargs.pop('pure', False)
-        return delayed(apply, pure=pure)(self, args, kwargs)
+        name = kwargs.pop('dask_key_name', None)
+        func = delayed(apply, pure=pure)
+        if name is not None:
+            return func(self, args, kwargs, dask_key_name=name)
+        return func(self, args, kwargs)
 
     def __bool__(self):
         raise TypeError("Truth of Delayed objects is not supported")
@@ -334,7 +373,7 @@ class DelayedLeaf(Delayed):
             try:
                 name = obj.__name__ + tokenize(obj, pure=pure)
             except AttributeError:
-                name = type(obj).__name__ + tokenize(obj, pure=pure)
+                name = '%s-%s' % (type(obj).__name__, tokenize(obj, pure=pure))
         object.__setattr__(self, '_dasks', [{name: obj}])
         object.__setattr__(self, 'pure', pure)
 
