@@ -13,6 +13,7 @@ import tempfile
 import inspect
 import codecs
 import math
+from importlib import import_module
 from sys import getdefaultencoding
 
 try:
@@ -62,6 +63,17 @@ def ignoring(*exceptions):
         pass
 
 
+def import_required(mod_name, error_msg):
+    """Attempt to import a required dependency.
+
+    Raises a RuntimeError if the requested module is not available.
+    """
+    try:
+        return import_module(mod_name)
+    except ImportError:
+        raise RuntimeError(error_msg)
+
+
 @contextmanager
 def tmpfile(extension='', dir=None):
     extension = '.' + extension.lstrip('.')
